/*
 * $Header: /home/orchestra5/davy/stuff/misc/xsat/RCS/commands.c,v 1.1 92/04/10 14:08:09 davy Exp $
 *
 * Copyright 1992 by David A. Curry
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that copyright
 * notice and this permission notice appear in supporting documentation.  The
 * author makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 *
 * Routines for dealing with the "commands" menu.
 *
 * David A. Curry
 * Purdue University
 * Engineering Computer Network
 * 1285 Electrical Engineering Building
 * West Lafayette, IN 47907
 * davy@ecn.purdue.edu
 *
 * $Log:	commands.c,v $
 * Revision 1.1  92/04/10  14:08:09  davy
 * Initial revision
 * 
 */
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Dialog.h>
#include <stdio.h>

#include "xsat.h"

static void	doPostScript();
static void	setStart();
static void	setStop();
static void	setStep();
static void	setElev();
static void	setTime();
static void	doPrint();
static void	doSave();

/*
 * Prompt the user for a new start time.
 */
void
commandsSetStartTime(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	prompt("Enter start time:", startTimeStr, setStart);
}

/*
 * Prompt the user for a new stop time.
 */
void
commandsSetStopTime(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	prompt("Enter stop time:", stopTimeStr, setStop);
}

/*
 * Prompt the user for a new step time.
 */
void
commandsSetStepTime(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	char buf[16];

	sprintf(buf, "%d", stepTime);
	prompt("Enter step time (seconds):", buf, setStep);
}

/*
 * Prompt the user for a new minimum elevation.
 */
void
commandsSetMinElev(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	char buf[16];

	sprintf(buf, "%d", minElevation);
	prompt("Enter minimum elevation (degrees):", buf, setElev);
}

/*
 * Prompt the user for a new time label frequency.
 */
void
commandsSetTimeLabels(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	char buf[16];

	sprintf(buf, "%d", timeLabelFreq);
	prompt("Enter time label frequency (minutes):", buf, setTime);
}

/*
 * Run the orbit prediction.
 */
void
commandsPredict(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	Arg argList[2];
	char buf[BUFSIZ];

	if (currentCity == NULL) {
		prompt("Must select a city first.", NULL, promptOkay);
		return;
	}

	if (currentSat == NULL) {
		prompt("Must select a satellite first.", NULL, promptOkay);
		return;
	}

	rewind(tmpFP);
	ftruncate(fileno(tmpFP), 0);

	/*
	 * Clear the scratch area if we're drawing.
	 */
	if (currentMap != NULL) {
		canvasClearScratch(mapCanvas);
		canvasUpdateWindow(mapCanvas);
	}

	canvasBusy(mapCanvas);

	/*
	 * Run the simulation.
	 */
	orbitPredict(mapsDrawLineX, mapsDrawTextX);

	/*
	 * Update the window if we drew.
	 */
	if (currentMap != NULL)
		canvasUpdateWindow(mapCanvas);

	fflush(tmpFP);

	/*
	 * Label stuff and reset the text widget.
	 */
	XtSetArg(argList[0], XtNstring, tmpFile);
	XtSetValues(orbitText, argList, ONE);

	sprintf(buf, "Orbit Information for %s from %s, %s",
		currentSat->s_name, currentCity->c_city,
		currentCity->c_statecountry);
	XtSetArg(argList[0], XtNlabel, buf);
	XtSetValues(botLabel, argList, ONE);

	canvasIdle(mapCanvas);
}

/*
 * Print the output.
 */
void
commandsPrint(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	if (currentCity == NULL) {
		prompt("Must select a city first.", NULL, promptOkay);
		return;
	}

	if (currentSat == NULL) {
		prompt("Must select a satellite first.", NULL, promptOkay);
		return;
	}

	prompt("Print command:", printCmd, doPrint);
}

/*
 * Save the output to a file.
 */
void
commandsSave(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	if (currentCity == NULL) {
		prompt("Must select a city first.", NULL, promptOkay);
		return;
	}

	if (currentSat == NULL) {
		prompt("Must select a satellite first.", NULL, promptOkay);
		return;
	}

	prompt("File name:", saveFile, doSave);
}

/*
 * Quit.
 */
void
commandsQuit(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	XtUnmapWidget(topLevel);
	XCloseDisplay(XtDisplay(topLevel));

	unlink(tmpFile);
	exit(0);
}

/*
 * Callback for setting start time.
 */
static void
setStart(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	char *new;
	Arg argList[2];

	new = XawDialogGetValueString(XtParent(w));

	startTime = getdate(new, NULL);
	new = topLabelStr(startTime, stopTime, stepTime);

	XtSetArg(argList[0], XtNlabel, new);
	XtSetValues(topLabel, argList, ONE);
	XtDestroyWidget(clientData);
}

/*
 * Callback for setting stop time.
 */
static void
setStop(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	char *new;
	Arg argList[2];

	new = XawDialogGetValueString(XtParent(w));

	stopTime = getdate(new, NULL);
	new = topLabelStr(startTime, stopTime, stepTime, minElevation);

	XtSetArg(argList[0], XtNlabel, new);
	XtSetValues(topLabel, argList, ONE);
	XtDestroyWidget(clientData);
}

/*
 * Callback for setting step time.
 */
static void
setStep(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	char *new;
	Arg argList[2];

	new = XawDialogGetValueString(XtParent(w));

	stepTime = atoi(new);
	new = topLabelStr(startTime, stopTime, stepTime, minElevation);

	XtSetArg(argList[0], XtNlabel, new);
	XtSetValues(topLabel, argList, ONE);
	XtDestroyWidget(clientData);
}

/*
 * Callback for setting minimum elevation.
 */
static void
setElev(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	char *new;
	Arg argList[2];

	new = XawDialogGetValueString(XtParent(w));

	minElevation = atoi(new);
	new = topLabelStr(startTime, stopTime, stepTime, minElevation);

	XtSetArg(argList[0], XtNlabel, new);
	XtSetValues(topLabel, argList, ONE);
	XtDestroyWidget(clientData);
}

/*
 * Callback for setting time label frequency.
 */
static void
setTime(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	char *new;
	Arg argList[2];

	new = XawDialogGetValueString(XtParent(w));

	timeLabelFreq = atoi(new);
	XtDestroyWidget(clientData);
}

/*
 * Callback for printing the file.
 */
static void
doPrint(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	strcpy(printCmd, XawDialogGetValueString(XtParent(w)));

	if ((psFP = popen(printCmd, "w")) == NULL) {
		prompt("Cannot open pipe to print command.", NULL, promptOkay);
		return;
	}

	canvasBusy(mapCanvas);

	doPostScript();
	pclose(psFP);

	XtDestroyWidget(clientData);
	canvasIdle(mapCanvas);
}

/*
 * Callback for saving the file.
 */
static void
doSave(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	strcpy(saveFile, XawDialogGetValueString(XtParent(w)));

	if ((psFP = fopen(saveFile, "w")) == NULL) {
		prompt("Cannot open save file.", NULL, promptOkay);
		return;
	}

	canvasBusy(mapCanvas);

	doPostScript();
	fclose(psFP);

	XtDestroyWidget(clientData);
	canvasIdle(mapCanvas);
}

/*
 * Generate PostScript output.
 */
static void
doPostScript()
{
	int nlines;
	char buf[BUFSIZ];
	float xscale, yscale;

	fprintf(psFP, "%%!\n0 setgray\n0 setlinewidth\n");
	fprintf(psFP, "/D { moveto lineto stroke } def\n");
	fprintf(psFP, "7.5 72 mul 0.5 72 mul translate\n90 rotate\n");

	/*
	 * If there's a map selected, print it first.
	 */
	if (currentMap != NULL) {
		if (currentMap->m_maxx < 0) {
			xscale = abs(currentMap->m_minx) -
				 abs(currentMap->m_maxx);
		}
		else {
			xscale = currentMap->m_maxx - currentMap->m_minx;
		}

		if (currentMap->m_maxy < 0) {
			yscale = abs(currentMap->m_miny) -
				 abs(currentMap->m_maxy);
		}
		else {
			yscale = currentMap->m_maxy - currentMap->m_miny;
		}

		fprintf(psFP, "gsave\n");
		fprintf(psFP, "10 72 mul %.4f div 7 72 mul %.4f div scale\n",
			xscale, yscale);
		fprintf(psFP, "/Helvetica findfont [%.4f 0 0 %.4f 0 0] ",
			8 / (720/xscale), 8 / (504/yscale));
		fprintf(psFP, "makefont setfont\n");

		mapsDrawMapPS();
	}

	rewind(tmpFP);
	ftruncate(fileno(tmpFP), 0);

	/*
	 * Run the orbit simulation.
	 */
	orbitPredict(mapsDrawLinePS, mapsDrawTextPS);

	fflush(tmpFP);

	/*
	 * Finish up the map, if there was one.
	 */
	if (currentMap != NULL) {
		fprintf(psFP, "grestore\n");
		fprintf(psFP, "/Helvetica findfont 10 scalefont setfont\n");
		fprintf(psFP, "0 -12 moveto (%s) show\n",
			currentSat->s_name);
		fprintf(psFP, "0 -24 moveto (NORAD catalog #%d) show\n",
			currentSat->s_satnum);
		fprintf(psFP, "/s1 (Start Time: %s) def\n", startTimeStr);
		fprintf(psFP, "/s2 (Stop Time: %s) def\n", stopTimeStr);
		fprintf(psFP, "10 72 mul s1 stringwidth pop sub -12 moveto\n");
		fprintf(psFP, "s1 show\n");
		fprintf(psFP, "10 72 mul s2 stringwidth pop sub -24 moveto\n");
		fprintf(psFP, "s2 show\n");
		fprintf(psFP, "showpage\n");
		fprintf(psFP, "7.5 72 mul 0.5 72 mul translate\n");
		fprintf(psFP, "90 rotate\n");
	}

	fprintf(psFP, "/Courier findfont 12 scalefont setfont\n");

	rewind(tmpFP);

	/*
	 * Copy in the orbit data.
	 */
	nlines = 0;
	while (fgets(buf, sizeof(buf), tmpFP) != NULL) {
		buf[strlen(buf)-1] = '\0';
		fprintf(psFP, "72 7.0 72 mul %d sub moveto\n", nlines * 14);
		fprintf(psFP, "(%s) show\n", buf);

		if (++nlines == 38) {
			fprintf(psFP, "showpage\n");
			fprintf(psFP, "7.5 72 mul 0.5 72 mul translate\n");
			fprintf(psFP, "90 rotate\n");
			nlines = 0;
		}
	}

	if (nlines > 0)
		fprintf(psFP, "showpage\n");
}
