/*
 * $Header: /home/orchestra5/davy/stuff/misc/xsat/RCS/util.c,v 1.1 92/04/10 14:08:15 davy Exp $
 *
 * Copyright 1992 by David A. Curry
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that copyright
 * notice and this permission notice appear in supporting documentation.  The
 * author makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 *
 * Utility routines.
 *
 * David A. Curry
 * Purdue University
 * Engineering Computer Network
 * 1285 Electrical Engineering Building
 * West Lafayette, IN 47907
 * davy@ecn.purdue.edu
 *
 * $Log:	util.c,v $
 * Revision 1.1  92/04/10  14:08:15  davy
 * Initial revision
 * 
 */
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Dialog.h>
#include <sys/timeb.h>
#include <stdio.h>
#include <time.h>

#include "xsat.h"

static Widget	dialog;

/*
 * Set up initial values for variables.
 */
void
setInitialValues()
{
	char *mktemp();

	stepTime = appResources.stepTime;
	minElevation = appResources.minElev;
	timeLabelFreq = appResources.timeLabelFreq;

	stopTime = getdate(appResources.stopTime, NULL);
	startTime = getdate(appResources.startTime, NULL);

	strcpy(printCmd, appResources.printCmd);
	strcpy(saveFile, appResources.saveFile);

	if (appResources.mapName != NULL) {
		if ((currentMap = findMap(appResources.mapName)) == NULL) {
			fprintf(stderr, "unknown map name: %s\n",
				appResources.mapName);
			exit(1);
		}
	}

	if (appResources.satName != NULL) {
		if ((currentSat = findSat(appResources.satName)) == NULL) {
			fprintf(stderr, "unknown satellite name: %s\n",
				appResources.satName);
			exit(1);
		}
	}

	if (appResources.cityName != NULL) {
		if ((currentCity = findCity(appResources.cityName)) == NULL) {
			fprintf(stderr, "unknown city name: %s\n",
				appResources.cityName);
			exit(1);
		}
	}

	tmpFile = mktemp("/tmp/xsatXXXXXX");
	tmpFP = fopen(tmpFile, "w+");
}

/*
 * Prompt a user for some input.
 */
char *
prompt(str, val, func)
char *str, *val;
void (*func)();
{
	Position x, y;
	Cardinal nargs;
	Arg argList[8];
	Widget w, popup;

	XtTranslateCoords(topLevel, 50, 50, &x, &y);

	nargs = 0;
	XtSetArg(argList[nargs], XtNx, x); nargs++;
	XtSetArg(argList[nargs], XtNy, y); nargs++;
	XtSetArg(argList[nargs], XtNinput, True); nargs++;
	XtSetArg(argList[nargs], XtNallowShellResize, True); nargs++;
	XtSetArg(argList[nargs], XtNtransientFor, topLevel); nargs++;
	popup = XtCreatePopupShell("prompt", transientShellWidgetClass,
				   topLevel, argList, nargs);

	nargs = 0;
	XtSetArg(argList[nargs], XtNlabel, str); nargs++;

	if (val != NULL) {
		XtSetArg(argList[nargs], XtNvalue, val);
		nargs++;
	}
	
	dialog = XtCreateManagedWidget("prompt", dialogWidgetClass, popup,
				       argList, nargs);

	XawDialogAddButton(dialog, "command", func, popup);

	if (val != NULL) {
		XtSetArg(argList[0], XtNresizable, True); nargs++;
		w = XtNameToWidget(dialog, "value");
		XtSetValues(w, argList, ONE);
	}

	XtPopup(popup, XtGrabNone);
}

/*
 * Make the top label string.
 */
char *
topLabelStr(start, stop, step, elev)
time_t start, stop;
int step, elev;
{
	char *ap;
	char *tzn;
	struct tm *tp;
	struct timeb tz;
	char tmp[BUFSIZ];
	static char str[BUFSIZ];

	ftime(&tz);

	tp = appResources.utc ? gmtime(&start) : localtime(&start);
	ap = asctime(tp);

	tzn = timezone(tz.timezone, tp->tm_isdst);
	ap[strlen(ap)-1] = '\0';

	sprintf(startTimeStr, "%.20s", ap);
	if (tzn != NULL)
		strcat(startTimeStr, tzn);
	strcat(startTimeStr, ap+19);

	sprintf(str, "Start: %s  Stop: ", startTimeStr);

	tp = appResources.utc ? gmtime(&stop) : localtime(&stop);
	ap = asctime(tp);

	ap[strlen(ap)-1] = '\0';

	sprintf(stopTimeStr, "%.20s", ap);
	if (tzn != NULL)
		strcat(stopTimeStr, tzn);
	strcat(stopTimeStr, ap+19);

	strcat(str, stopTimeStr);

	sprintf(tmp, "  Step: %d sec  MinElev: %d deg", step, elev);
	strcat(str, tmp);

	return(str);
}

/*
 * Callback for prompt().
 */
void
promptOkay(w, clientData, callData)
XtPointer clientData, callData;
Widget w;
{
	XtDestroyWidget(clientData);
}

/*
 * Strip trailing spaces from a string.
 */
void
stripSpace(s)
char *s;
{
	char *t;

	t = &s[strlen(s)-1];

	while (((*t == ' ') || (*t == '\t')) && (t > s))
		t--;

	*++t = '\0';
}

/*
 * Save a string in alloc'd memmory.
 */
char *
saveString(s)
char *s;
{
	char *t;
	char *malloc();

	if ((t = malloc(strlen(s)+1)) == NULL) {
		fprintf(stderr, "out of memory.\n");
		exit(1);
	}

	strcpy(t, s);
	return(t);
}

/*
 * Allocate memory with error checking.
 */
char *
safeAlloc(size)
int size;
{
	char *p;

	if ((p = malloc(size)) == NULL) {
		fprintf(stderr, "out of memory.\n");
		exit(1);
	}

	return(p);
}
